/*
  Copyright 2012  Damian Murphy <murff@warlock.org>

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package org.warlock.itk.PHXMLadapter.commandline;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.BufferedReader;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.InputSource;
import java.io.StringReader;

import org.w3c.dom.Node;
import org.warlock.itk.PHXMLadapter.phxmlconverter;
/**
 * Simple command line caller for the PH <-> XML adapter. To convert XML to PH:
 * 
 * java -jar <jarfile> -xp infile outfile
 * 
 * To convert PH to XML:
 * 
 * java -jar <jarfile> -px infile outfile
 * 
 * @author Damian Murphy <murff@warlock.org>
 */
public class PHXMLadapter {

    /**
     * Version: PHXMLadapter v1.0.0-Open
     */
    public static final String VERSION = "PHXMLadapter v1.0.0-Open";
    public static final String USAGE = "[-h] -px | -xp infile outfile";
    private static final int OP_PHTOXML = -1;
    private static final int OP_XMLTOPH = 1;
    private static final int OP_UNDEFINED = 0;
    
    public static void main(String[] args) {
        int op = OP_UNDEFINED;
        int a = 0;
        try {
            if (args[a].contentEquals("-h")) {
                System.out.println(VERSION);
                System.out.println(USAGE);
                a++;
            }
            if ((args.length == 1) && (a == 1)) {
                System.exit(0);
            }
            if (args[a].contentEquals("-px")) {
                op = OP_PHTOXML;
            } else {
                if (args[a].contentEquals("-xp")) {
                    op = OP_XMLTOPH;
                } else {
                    System.err.println("Invalid arguments, should be: " + USAGE);
                    System.exit(1);
                }
            }
            if (op == OP_UNDEFINED) {
                System.err.println("Invalid arguments, should be: " + USAGE);
                System.exit(1);                
            }
            String infile = loadFile(args[a + 1]);
            String out = null;
            phxmlconverter p = phxmlconverter.getInstance();
            switch (op) {
                case OP_PHTOXML:
                    String mtype = phxmlconverter.findMessageType(infile);
                    out = p.convert(mtype, infile);
                    break;
                    
                case OP_XMLTOPH:
                    Element xml = parse(infile);
                    if (xml != null) {
                        out = p.convert(xml);
                    }
                    break;
            }
            if (out != null) {
                try {
                    FileWriter fw = new FileWriter(args[a + 2]);
                    fw.write(out);
                    fw.flush();
                    fw.close();
                }
                catch (Exception e) {
                    throw new Exception("Error writing output " + args[a + 2] + " : " + e.toString());
                }
            }
        }
        catch (Exception e) {
//            e.printStackTrace();
//            System.err.println("Error converting: " + e.getMessage());
            System.err.println(e.getMessage());
        }
    }
    
    private static Element parse(String in)
            throws Exception
    {
        try {
            InputSource is = new InputSource(new StringReader(in));
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document d = db.parse(is);
            NodeList nl = d.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                Node n = nl.item(i);
                if (n.getNodeType() == Node.ELEMENT_NODE) {
                    return (Element)n;
                }
            }
            return null;
        }
        catch (Exception e) {
            throw new Exception("Error parsing input: " + e.toString());
        }
    }
    
    private static String loadFile(String filename) 
            throws Exception
    {
        try {
            FileReader fr = new FileReader(filename);
            BufferedReader br = new BufferedReader(fr);
            StringBuilder sb = new StringBuilder();
            String line = null;
            while((line = br.readLine()) != null) {
                sb.append(line);
                sb.append("\n");
            }
            fr.close();
            return sb.toString();
        }
        catch (Exception e) {
            throw new Exception("Error reading file " + filename + " : " + e.toString());
        }
    }
}
